//
//  Top3TableViewController.swift
//  Game
//
//  Created by Santiago Pavón on 8/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit

let GAME_URL = "http://localhost:3000/scores/top3.json"

class Top3TableViewController: UITableViewController {

    // Los datos descargados:
    var top3Scores: [[String:Any]] = []
    
    // La session
    var session: URLSession!
    
    @IBOutlet weak var refreshButton: UIBarButtonItem!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Crear la session
        let config = URLSessionConfiguration.default
        session = URLSession(configuration: config)
       
        // Descarga inicial de datos
        getTop3()
    }

    // MARK: Acciones
    
    @IBAction func getTop3() {
        // v1()
        v2()
    }
    
    fileprivate func v1() {
    
        // Construir la URL
        let url = URL(string: GAME_URL)!
        
        // Crear Data Task
        let dataTask = session.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
            
            if error != nil {
                DispatchQueue.main.async {
                    print("Error descargando: \(error!.localizedDescription)")
                    self.title = "Desactualizado"
                }
            } else if let res = response as? HTTPURLResponse, res.statusCode != 200 {
                DispatchQueue.main.async {
                    let msg = HTTPURLResponse.localizedString(forStatusCode: res.statusCode)
                    print("Error descargando: \(msg)")
                    self.title = "Desactualizado"
                }
            } else {
                do {
                    let newTop3 = (try JSONSerialization.jsonObject(with: data!)) as? [[String:Any]]
                    DispatchQueue.main.async {
                        self.top3Scores = newTop3!
                        self.tableView.reloadData()
                        self.title = "Top 3"
                    }
                } catch let err {
                    DispatchQueue.main.async {
                        print("Error extrayendo JSON = \(err.localizedDescription)")
                        self.title = "Desactualizado"
                    }
                }
            }
        }
        dataTask.resume()
    }
    
    fileprivate func v2() {
        title = "Descargando..."
        refreshButton.isEnabled = false
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
        
        // Construir la URL
        let url = URL(string: GAME_URL)!
        
        // Crear Data Task
        let dataTask = session.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) in
            
            var newTop3: [[String:Any]]?
            
            if error == nil,
                (response as? HTTPURLResponse)?.statusCode == 200 {
                newTop3 = (try? JSONSerialization.jsonObject(with: data!)) as? [[String:Any]]
            }
            
            // El completionHandler no corre en el Main Thread
            DispatchQueue.main.async {
                if newTop3 == nil {
                    self.title = "ERROR"
                } else {
                    self.top3Scores = newTop3!
                    self.tableView.reloadData()
                    self.title = "Top 3"
                }
                self.refreshButton.isEnabled = true
                UIApplication.shared.isNetworkActivityIndicatorVisible = false
            }
        }
        dataTask.resume()
    }
    
    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return top3Scores.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "Score Cell", for: indexPath) 

        let dic = top3Scores[indexPath.row]

        cell.textLabel?.text = dic["name"] as? String
        let total = dic["total"] as! Int
        cell.detailTextLabel?.text = "\(total)"
        
        return cell
    }

}
